
import thread
import subprocess
import os
from gi.repository import GObject

RFKILL_CHK = ["/usr/sbin/rfkill", "list", "bluetooth"]
RFKILL_BLOCK = ["/usr/sbin/rfkill", "block", "bluetooth"]
RFKILL_UNBLOCK = ["/usr/sbin/rfkill", "unblock", "bluetooth"]

RFKILL_EVENT_MONITOR = ["/usr/sbin/rfkill", "event"]

# index of .split() from rfkill event output where lines are:
#     1426095957.906704: idx 0 type 1 op 0 soft 0 hard 0
#     1426095957.906769: idx 1 type 2 op 0 soft 1 hard 0
#     1426096013.465033: idx 1 type 2 op 2 soft 0 hard 0

EVENT_INDEX_DEVICE = 2
EVENT_INDEX_SOFT_BLOCK = 8
EVENT_INDEX_HARD_BLOCK = 10

class Interface:
    def __init__(self, output_callback, debug):
        self.enable_debugging = debug
        self.output_callback = output_callback
        self.have_adapter = False
        self.adapter_index = -1

        self.tproc = None
        self.blockproc = None

        self.hard_block = False
        self.soft_block = False

        self.monitor_killer = False

        self.have_adapter = self.adapter_check()

        if self.have_adapter:
            self.start_event_monitor()

    def adapter_check(self):
        res = subprocess.check_output(RFKILL_CHK)

        '''
        Assume the output of:

        > /usr/sbin/rfkill list bluetooth

        looks like:

        1: hci0: Bluetooth
            Soft blocked: yes
            Hard blocked: no
        '''

        self.debug("adapter_check full output:\n%s" % res)

        if not res:
            self.debug("adapter_check no output (no adapter)")
            self.have_adapter = False
            return False

        reslines = res.split('\n')
        for line in reslines:
            if "Bluetooth" in line:
                self.adapter_index = int(line[0])
                self.debug("adapter_check found adapter at %d" % self.adapter_index)
                return True

        return False

    def start_event_monitor(self):
        if not self.tproc and not self.monitor_killer:
            thread.start_new_thread(self.event_monitor_thread, (None,))

    def event_monitor_thread(self, data):
        self.tproc = subprocess.Popen(RFKILL_EVENT_MONITOR, stdout=subprocess.PIPE)
        while self.tproc.poll() is None and not self.monitor_killer:
            l = self.tproc.stdout.readline() # This blocks until it receives a newline.
            self.update_state(l)

        self.tproc = None
        thread.exit()

    def update_state(self, line):
        self.debug("update_state line: %s" % line)

        elements = line.split()

        if len(elements) < EVENT_INDEX_HARD_BLOCK:
            return

        if int(elements[EVENT_INDEX_DEVICE]) == self.adapter_index:
            self.soft_block = int(elements[EVENT_INDEX_SOFT_BLOCK]) == 1
            self.hard_block = int(elements[EVENT_INDEX_HARD_BLOCK]) == 1

        self.update_ui()

    def update_ui(self):
        GObject.idle_add(self.output_callback)

    def try_set_blocked(self, blocked):
        thread.start_new_thread(self.set_block_thread, (blocked,))

    def set_block_thread(self, data):
        block = data

        if block:
            self.debug("set_block_thread blocking")
            self.blockproc = subprocess.Popen(RFKILL_BLOCK)
        else:
            self.debug("set_block_thread unblocking")
            self.blockproc = subprocess.Popen(RFKILL_UNBLOCK)

        while self.blockproc.poll() is None:
            pass

        self.blockproc = None
        self.debug("set_block_thread finished")
        thread.exit()

    def terminate(self):
        if self.tproc:
            self.tproc.kill()
        if self.blockproc:
            self.blockproc.kill()

    def debug(self, msg):
        if self.enable_debugging:
            print(msg)




#!/usr/bin/env python2

import sys, os, commands
import gettext
import rfkillMagic
import subprocess
import blueberrySettings

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('GnomeBluetooth', '1.0')
from gi.repository import Gtk, GnomeBluetooth, Gio


BLUETOOTH_DISABLED_PAGE      = "disabled-page"
BLUETOOTH_HW_DISABLED_PAGE   = "hw-disabled-page"
BLUETOOTH_NO_DEVICES_PAGE    = "no-devices-page"
BLUETOOTH_WORKING_PAGE       = "working-page"

# i18n
gettext.install("blueberry", "/usr/share/locale")

# detect the DE environment
wm_info = commands.getoutput("wmctrl -m")
if "XDG_CURRENT_DESKTOP" in os.environ:
    xdg_current_desktop = os.environ["XDG_CURRENT_DESKTOP"]
else:
    xdg_current_desktop = ""

if "Marco" in wm_info or xdg_current_desktop == "MATE":
    CONF_TOOLS = {"sound": "mate-volume-control", "keyboard": "mate-keyboard-properties", "mouse": "mate-mouse-properties"}
elif "Xfwm4" in wm_info or xdg_current_desktop == "XFCE":
    CONF_TOOLS = {"keyboard": "xfce4-keyboard-settings", "mouse": "xfce4-mouse-settings"}
    if os.path.exists("/usr/bin/pavucontrol"):
        CONF_TOOLS["sound"] = "pavucontrol"
    else:
        CONF_TOOLS["sound"] = "xfce4-mixer"
elif "Muffin" in wm_info or xdg_current_desktop == "X-Cinnamon":
    CONF_TOOLS = {"sound": "cinnamon-settings sound", "keyboard": "cinnamon-settings keyboard", "mouse": "cinnamon-settings mouse"}
elif "Mutter" in wm_info or "GNOME" in xdg_current_desktop:
    CONF_TOOLS = {"sound": "gnome-control-center sound", "keyboard": "gnome-control-center keyboard", "mouse": "gnome-control-center mouse"}
elif "Unity" in wm_info or xdg_current_desktop == "Unity":
    CONF_TOOLS = {"sound": "unity-control-center sound", "keyboard": "unity-control-center keyboard", "mouse": "unity-control-center mouse"}
elif xdg_current_desktop == "LXDE":
    CONF_TOOLS = {"sound": "pavucontrol", "keyboard": "lxinput", "mouse": "lxinput"}
else:
    print "Warning: DE could not be detected!"
    CONF_TOOLS = {}
 
class Blueberry(Gtk.Application):
    ''' Create the UI '''
    def __init__(self):

        Gtk.Application.__init__(self, application_id='com.linuxmint.blueberry', flags=Gio.ApplicationFlags.FLAGS_NONE)
        self.connect("activate", self.on_activate)

    def on_activate(self, data=None):
        list = self.get_windows()
        if len(list) > 0:
            # Blueberry is already running, focus the window
            self.get_active_window().present()
        else:
            self.create_window()
            
    def create_window(self):
        self.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)

        self.window.set_title(_("Bluetooth"))
        self.window.set_icon_name("bluetooth")
        self.window.connect("destroy", self.terminate)
        self.window.set_default_size(640, 480)

        self.main_box = Gtk.VBox()

        self.status_image = Gtk.Image()
        self.status_image.set_from_icon_name("blueberry", Gtk.IconSize.DIALOG)
        self.status_image.show()

        self.stack = Gtk.Stack()
        self.rf_switch = Gtk.Switch()

        self.add_stack_page(_("Bluetooth is disabled"), BLUETOOTH_DISABLED_PAGE);
        self.add_stack_page(_("No Bluetooth adapters found"), BLUETOOTH_NO_DEVICES_PAGE);
        self.add_stack_page(_("Bluetooth is disabled by hardware switch"), BLUETOOTH_HW_DISABLED_PAGE);

        self.lib_widget = GnomeBluetooth.SettingsWidget.new();

        self.lib_widget.connect("panel-changed", self.panel_changed);

        self.stack.add_named(self.lib_widget, BLUETOOTH_WORKING_PAGE)

        self.lib_widget.show()
        self.stack.show();
        self.main_box.show();
        self.main_box.set_border_width(12)

        switchbox = Gtk.VBox()
        hbox = Gtk.HBox()
        hbox.pack_end(self.rf_switch, False, False, 0)
        switchbox.pack_start(hbox, False, False, 0)
        switchbox.pack_start(self.status_image, False, False, 10)
        switchbox.show_all()

        self.main_box.pack_start(switchbox, False, False, 10)
        self.main_box.pack_start(self.stack, True, True, 10)

        self.window.add(self.main_box)

        debug = False
        if len(sys.argv) > 1 and sys.argv[1] == "debug":
            debug = True

        self.rfkill = rfkillMagic.Interface(self.update_ui_callback, debug)
        self.rf_handler_id = self.rf_switch.connect("state-set", self.on_switch_changed)

        self.settings = blueberrySettings.Settings()

        traybox = Gtk.HBox()
        self.traybutton = Gtk.CheckButton(label=_("Show a tray icon"))
        self.traybutton.set_active(self.settings.get_tray_enabled())
        self.traybutton.connect("toggled", self.on_tray_button_toggled)
        self.settings.gsettings.connect("changed::tray-enabled", self.on_settings_changed)

        traybox.pack_start(self.traybutton, False, False, 0)
        traybox.show_all()

        self.main_box.pack_start(traybox, False, False, 0)

        self.window.show()

        self.update_ui_callback()

        self.add_window(self.window)

    def panel_changed(self, widget, panel):
        if not panel in CONF_TOOLS:
            print "Warning, no configuration tool known for panel '%s'" % panel
        else:
            os.system("%s &" % CONF_TOOLS[panel])

    def on_tray_button_toggled(self, widget, data=None):
        if widget.get_active():
            self.settings.set_tray_enabled(True)
            subprocess.Popen(["blueberry-tray"])
        else:
            self.settings.set_tray_enabled(False)

    def on_settings_changed(self, settings, key):
        self.traybutton.set_active(self.settings.get_tray_enabled())

    def add_stack_page(self, message, name):
        label = Gtk.Label(message)
        self.stack.add_named(label, name)
        label.show()

    def update_ui_callback(self):
        powered = False
        sensitive = False
        page = ""

        if not self.rfkill.have_adapter:
            page = BLUETOOTH_NO_DEVICES_PAGE
            self.status_image.set_from_icon_name("blueberry-disabled", Gtk.IconSize.DIALOG)
        elif self.rfkill.hard_block:
            page = BLUETOOTH_HW_DISABLED_PAGE
            self.status_image.set_from_icon_name("blueberry-disabled", Gtk.IconSize.DIALOG)
        elif self.rfkill.soft_block:
            page = BLUETOOTH_DISABLED_PAGE
            sensitive = True
            self.status_image.set_from_icon_name("blueberry-disabled", Gtk.IconSize.DIALOG)
        else:
            page = BLUETOOTH_WORKING_PAGE
            sensitive = True
            powered = True
            self.status_image.set_from_icon_name("blueberry", Gtk.IconSize.DIALOG)

        self.rf_switch.set_sensitive(sensitive)

        self.rf_switch.handler_block(self.rf_handler_id)
        self.rf_switch.set_state(powered)
        self.rf_switch.handler_unblock(self.rf_handler_id)

        self.stack.set_visible_child_name (page);

    def on_switch_changed(self, widget, state):
        self.rfkill.try_set_blocked(not state)
        return True

    def terminate(self, window):
        self.rfkill.terminate()

if __name__ == "__main__":
    app = Blueberry()
    app.run(None)



#!/usr/bin/env python2

from gi.repository import Gio

SETTINGS_SCHEMA = "org.blueberry"
TRAY_KEY = "tray-enabled"

class Settings():
    def __init__(self):
        self.gsettings = Gio.Settings.new(SETTINGS_SCHEMA)

    def get_tray_enabled(self):
        return self.gsettings.get_boolean(TRAY_KEY)

    def set_tray_enabled(self, enabled):
        self.gsettings.set_boolean(TRAY_KEY, enabled)






#!/usr/bin/env python2

import sys
import gettext
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('GnomeBluetooth', '1.0')
from gi.repository import Gtk, Gdk, GnomeBluetooth, Gio
import rfkillMagic
import blueberrySettings
import subprocess

# i18n
gettext.install("blueberry", "/usr/share/locale")

class BluetoothTray:
    def __init__(self):
        debug = False
        if len(sys.argv) > 1 and sys.argv[1] == "debug":
            debug = True

        self.rfkill = rfkillMagic.Interface(self.update_icon_callback, debug)
        self.settings = blueberrySettings.Settings()
        self.settings.gsettings.connect("changed::tray-enabled", self.on_settings_changed_cb)

        # If we have no adapter, or disabled tray, end early
        if (not self.rfkill.have_adapter) or (not self.settings.get_tray_enabled()):
            self.rfkill.terminate()
            sys.exit(0)

        self.client = GnomeBluetooth.Client()
        self.model = self.client.get_model()
        self.model.connect('row-changed', self.update_icon_callback)
        self.model.connect('row-deleted', self.update_icon_callback)
        self.model.connect('row-inserted', self.update_icon_callback)

        self.icon = Gtk.StatusIcon()
        self.icon.set_title(_("Bluetooth"))
        self.icon.connect("popup-menu", self.on_popup_menu)
        self.icon.connect("activate", self.on_activate)

        self.update_icon_callback(None, None, None)

    def on_settings_changed_cb(self, setting, key, data=None):
        if not self.settings.get_tray_enabled():
            self.terminate()

    def update_icon_callback(self, path=None, iter=None, data=None):
        if not self.rfkill.have_adapter:
            self.terminate(None)
            return

        if self.rfkill.hard_block or self.rfkill.soft_block:
            self.icon.set_from_icon_name("blueberry-tray-disabled")
            self.icon.set_tooltip_text(_("Bluetooth"))
        else:
            self.icon.set_from_icon_name("blueberry-tray")
            self.update_connected_state()

    def update_connected_state(self):
        n_devices = self.get_n_devices()

        if n_devices > 0:
            self.icon.set_from_icon_name("blueberry-tray-active")
            self.icon.set_tooltip_text(gettext.ngettext("Bluetooth: %d device connected" % n_devices, 
                                                        "Bluetooth: %d devices connected" % n_devices,
                                                        n_devices))
        else:
            self.icon.set_from_icon_name("blueberry-tray")
            self.icon.set_tooltip_text(_("Bluetooth"))

    def get_n_devices(self):
        default_iter = None

        iter = self.model.get_iter_first()

        while iter:
            default = self.model.get_value(iter, GnomeBluetooth.Column.DEFAULT)
            if default:
                default_iter = iter
                break
            iter = self.model.iter_next(iter)

        if default_iter == None:
            return False

        n_devices = 0

        iter = self.model.iter_children(default_iter)
        while iter:
            connected = self.model.get_value(iter, GnomeBluetooth.Column.CONNECTED)
            if connected:
                n_devices += 1

            iter = self.model.iter_next(iter)

        return n_devices

    def on_activate(self, icon, data=None):
        subprocess.Popen(["blueberry"])

    def on_popup_menu(self, icon, button, time, data = None):
        menu = Gtk.Menu()

        def position_menu_cb(m, x, y=None, i=None):
            try:
                return Gtk.StatusIcon.position_menu(menu, x, y, icon)
            except (AttributeError, TypeError):
                return Gtk.StatusIcon.position_menu(menu, icon)

        if not(self.rfkill.hard_block or self.rfkill.soft_block):
            item = Gtk.MenuItem(label=_("Send files to a device"))
            item.connect("activate", self.send_files_cb)
            menu.append(item)

        item = Gtk.MenuItem(label=_("Open Bluetooth device manager"))
        item.connect("activate", self.open_manager_cb)
        menu.append(item)

        menu.append(Gtk.SeparatorMenuItem())

        item = Gtk.MenuItem(label=_("Quit"))
        item.connect("activate", self.terminate)
        menu.append(item)

        menu.show_all()

        device = Gdk.Display.get_default().get_device_manager().get_client_pointer()
        menu.popup_for_device(device, None, None, position_menu_cb, icon, button, time)

    def send_files_cb(self, item, data = None):
        subprocess.Popen(["bluetooth-sendto"])

    def open_manager_cb(self, item, data = None):
        subprocess.Popen(["blueberry"])

    def terminate(self, window = None, data = None):
        self.rfkill.terminate()
        Gtk.main_quit()

if __name__ == "__main__":
    BluetoothTray()
    Gtk.main()
